""" 
This is the base agent for cabot4
It's paired with an environment that has the agent's orientation 0,90,180 
or 270 and position x,y.  There's also a cognitive map. 

This is the basic place for the description of this particular CABot4 system 
in the code.  There are two objects; object 0 is the apple, and object 1
the banana.

You can't call this to run the agent.  You need either the spinnaker version
or the nest version that uses this.
"""

import sys
import time

from nealCoverClass import NealCoverFunctions
from stateMachineClass import FSAHelperFunctions
from parseClass import ParseClass
import cabot4InputLang as languageToDefine
from cABot4PlanClass import PlanClass
from cogmapClass import cogmapBaseClass
import generatorFile as generator

#psuedo class globals
numberPlanActions = -1;
fsa = -1
sim = -1

def init(SIM,neal,FSA):
    global fsa
    global sim 
    #print ("spin or nest")
    sim = SIM
    sim.setup(timestep=neal.DELAY,
                    min_delay=neal.DELAY,
                    max_delay=neal.DELAY, debug=0)
    fsa = FSA

def createInputs(startTime):
    inputSpikeTimes0 = [10.0 + startTime] 
    inputSpikeTimes1 = [50.0 + startTime]
    inputSpikeTimes2 = [100.0 + startTime]
    inputSpikeTimes3 = [150.0 + startTime]
    inputSpikeTimes4 = [200.0 + startTime]
    inputSpikeTimes5 = [250.0 + startTime]
    inputSpikeTimes6 = [300.0 + startTime]
    stopSpikeTimes = [400.0 + startTime]
    
    spikeArray0 = {'spike_times': [inputSpikeTimes0]}
    spikeArray1 = {'spike_times': [inputSpikeTimes1]}
    spikeArray2 = {'spike_times': [inputSpikeTimes2]}
    spikeArray3 = {'spike_times': [inputSpikeTimes3]}
    spikeArray4 = {'spike_times': [inputSpikeTimes4]}
    spikeArray5 = {'spike_times': [inputSpikeTimes5]}
    spikeArray6 = {'spike_times': [inputSpikeTimes6]}
    stopSpikeArray = {'spike_times': [stopSpikeTimes]}
    spikeGen0=sim.Population(1,sim.SpikeSourceArray,spikeArray0,
                                   label='inputSpikes_0')
    spikeGen1=sim.Population(1, sim.SpikeSourceArray, spikeArray1,
                                   label='inputSpikes_1')
    spikeGen2=sim.Population(1, sim.SpikeSourceArray, spikeArray2,
                                   label='inputSpikes_2')
    spikeGen3=sim.Population(1, sim.SpikeSourceArray, spikeArray3,
                                   label='inputSpikes_3')
    spikeGen4=sim.Population(1, sim.SpikeSourceArray, spikeArray4,
                                   label='inputSpikes_4')
    spikeGen5=sim.Population(1, sim.SpikeSourceArray, spikeArray5,
                                   label='inputSpikes_5')
    spikeGen6=sim.Population(1, sim.SpikeSourceArray, spikeArray6,
                                   label='inputSpikes_6')
    stopSpikeGen=sim.Population(1, sim.SpikeSourceArray, stopSpikeArray,
                                   label='stopSpikes')

    return [spikeGen0,spikeGen1,spikeGen2,spikeGen3,spikeGen4,spikeGen5,
            spikeGen6,stopSpikeGen]


def connectOneFinalParseStateAndOneGoal(parser,planner,parseState,goalNumber):
    goalCells = planner.getGoalCells()
    parseStateCells = parser.stateCells

    parser.parseStateStartsState(parseState,goalCells, goalNumber)
    fsa.stateTurnsOffState(goalCells,goalNumber,parseStateCells,parseState)

def connectParserAndPlanner(parser,planner):
    #We should probably change the parse structure so that we subclass
    #parseClass in cabot4InputLang, but we fill it in manually here for now.
    tLFinalState = 3   #turn left
    tRFinalState = 5
    sFFinalState = 8
    sBFinalState = 10  #15 is for where is the apple?
    mLFinalState = 20  #move left
    mRFinalState = 22  #move right

    connectOneFinalParseStateAndOneGoal(parser,planner,tLFinalState,
                                        planner.goalTurnLeft)
    connectOneFinalParseStateAndOneGoal(parser,planner,tRFinalState,
                                        planner.goalTurnRight)
    connectOneFinalParseStateAndOneGoal(parser,planner,sFFinalState,
                                        planner.goalStepForward)
    connectOneFinalParseStateAndOneGoal(parser,planner,sBFinalState,
                                        planner.goalStepBackward)
    connectOneFinalParseStateAndOneGoal(parser,planner,mLFinalState,
                                        planner.goalForwardAndLeft)
    connectOneFinalParseStateAndOneGoal(parser,planner,mRFinalState,
                                        planner.goalForwardAndRight)

#set up the blank topology with no binding.
def buildCognitiveMap(cogmap):
    cogmap.createObjects(2)
    cogmap.connectObjects()
    cogmap.createPlaces(4)
    cogmap.connectPlaces()
    cogmap.setupCogMapRecording()
    cogmap.makeLearningSynapses()
    cogmap.connectAutomaton()

    inputSpikeTime = [10]
    spikeArray = {'spike_times': [inputSpikeTime]}
    spikeSource=sim.Population(1,sim.SpikeSourceArray,spikeArray,
                                   label='cMStartSpikes')

    cogmap.sourceStartsAutomaton(spikeSource)
    
def testPlanCogMap(planner):
    inputSpikeTime2 = [100]
    spikeArray2 = {'spike_times': [inputSpikeTime2]}
    spikeSource2=sim.Population(1,sim.SpikeSourceArray,spikeArray2,
                                   label='planTestBind')
    #planner.testSetGoal(planner.goalStepForward,spikeSource2)
    #planner.testSetGoal(planner.goalPutYBAt1,spikeSource2)
    #1planner.testSetGoal(planner.goalPutYBAt2,spikeSource2)
    #planner.testSetGoal(planner.goalPutRAAt1,spikeSource2)

    inputSpikeTime3 = [600]
    spikeArray3 = {'spike_times': [inputSpikeTime3]}
    spikeSource3=sim.Population(1,sim.SpikeSourceArray,spikeArray3,
                                   label='planQueryBind')
    #planner.testSetGoal(planner.goalQueryYB,spikeSource3)
    #planner.testSetGoal(planner.goalQueryRA,spikeSource3)
    #planner.testSetGoal(planner.goalQuery1,spikeSource3)
    #planner.testSetGoal(planner.goalQuery2,spikeSource3)

def makeFillCogMapSpikeSources(inputTimes):
        inputSpikeTimes1 = inputTimes[0]
        inputSpikeTimes2 = inputTimes[1]
        spikeArray1 = {'spike_times': [inputSpikeTimes1]}
        spikeGen1=sim.Population(1,sim.SpikeSourceArray,spikeArray1,
                                   label='inputSpikes_1')
        spikeArray2 = {'spike_times': [inputSpikeTimes2]}
        spikeGen2=sim.Population(1,sim.SpikeSourceArray,spikeArray2,
                                   label='inputSpikes_2')
        return [spikeGen1,spikeGen2]


def fillCogMap(cogmap):
    spikeSources = makeFillCogMapSpikeSources([50.0,550.0])

    objectA = 1
    objectB = 0
    #fTest1 = True
    fTest1 = False
    if (fTest1):
        placeA = 0
        placeB = 3
    else :
        placeA = 2
        placeB = 0

    
    if (placeA == placeB):
        print ("bad places for bind")
    if (objectA == objectB):
        print ("bad objects for bind")
    #put object A on place A
    cogmap.sourceTurnsOnPlaceOn(placeA,spikeSources[0])
    cogmap.sourceTurnsOnObjectOn(objectA,spikeSources[0])
    cogmap.sourceTurnsOnBind(spikeSources[0])
    
    #put object B on place B
    cogmap.sourceTurnsOnPlaceOn(placeB,spikeSources[1])
    cogmap.sourceTurnsOnObjectOn(objectB,spikeSources[1])
    cogmap.sourceTurnsOnBind(spikeSources[1])
    

global SentenceFSAs
global WordPop
global PhrasePop

def setupGenerator():
    global SentenceFSAs
    global WordPop
    global PhrasePop

    ###read in sentence file
    filename="cabot4GenerateSentences"
    AllLines=generator.readFile(filename)

    ##create a population of neurons, where each neuron represents one word
    WordPopInfo=generator.CreateWordPop(fsa,AllLines)

    ####extract the phrases and sentence templates from a file
    StoredSenAndPhrasesInfo=generator.ExtractSentencesAndPhrases(AllLines) 

    WordPop=WordPopInfo[0]  ##word neurons
    #print (WordPopInfo[1])

    ##create a CA for each phrase
    PhraseCAs=generator.createPhraseCAs(StoredSenAndPhrasesInfo,fsa) 
    PhrasePop = PhraseCAs[0]

    ##create an FSA for each sentence
    #################CREATE FSA For each sentence
    [SentenceFSAs,NumStatesEachSen]=generator.createFSASentences(WordPopInfo,
                    fsa,PhraseCAs,StoredSenAndPhrasesInfo)
    generator.setupRecording(SentenceFSAs,WordPop,PhraseCAs[0])

    return [NumStatesEachSen,StoredSenAndPhrasesInfo,WordPop]

def printGenResults(fileName):
    sentenceCells = SentenceFSAs
    wordCells = WordPop
    phraseCells = PhrasePop
    generator.printResults(SentenceFSAs,WordPop,PhrasePop,'results/')

def testGenerator(genObj):
    numStates = genObj[0]
    storedSen = genObj[1]
    #Sentence="The apple is there."
    Sentence="It is in room 1."
    generator.TurnOnSentence(Sentence,SentenceFSAs,numStates,fsa,storedSen)

#This can be used to test the result of a parse.
def connectParserAndGenerator(parser,genObj):
    wAFinalState = 15

    numStates = genObj[0]
    storedSen = genObj[1]
    #Sentence="The apple is there."
    Sentence="It is in room 4."
    parserStateCells = parser.getStateCells()
    
    generator.stateStartsSentence(parserStateCells,wAFinalState,Sentence,SentenceFSAs,numStates,fsa,storedSen)

def connectABindPair (parseStateCells,cogMap,finalParseState,room,object):
    cogMap.fsa.stateTurnsOnState(parseStateCells,finalParseState,
                                 cogMap.placeBindOnCells,room)
    #And pick the object.
    cogMap.fsa.stateTurnsOnState(parseStateCells,finalParseState,
                                 cogMap.objectBindOnCells,object)
    cogMap.fsa.stateHalfTurnsOnState(parseStateCells,finalParseState,
                                     cogMap.automatonCells,1)#1 tryBindState
    cogMap.fsa.stateTurnsOffState(cogMap.automatonCells,4,parseStateCells,
                                  finalParseState)#4 bindDoneState
    
def connectBind (parser,cogMap):
    aR1FinalState = 41
    aR2FinalState = 43
    aR3FinalState = 45
    aR4FinalState = 47
    bR1FinalState = 52
    bR2FinalState = 54
    bR3FinalState = 56
    bR4FinalState = 58

    parseStateCells = parser.getStateCells()
    #connect the put the apple room commands in.
    connectABindPair(parseStateCells,cogMap,aR1FinalState,0,0)
    connectABindPair(parseStateCells,cogMap,aR2FinalState,1,0)
    connectABindPair(parseStateCells,cogMap,aR3FinalState,2,0)
    connectABindPair(parseStateCells,cogMap,aR4FinalState,3,0)
    #connect the put the banana room commands in.
    connectABindPair(parseStateCells,cogMap,bR1FinalState,0,1)
    connectABindPair(parseStateCells,cogMap,bR2FinalState,1,1)
    connectABindPair(parseStateCells,cogMap,bR3FinalState,2,1)
    connectABindPair(parseStateCells,cogMap,bR4FinalState,3,1)
    

def connectAnUnbindPair (parseStateCells,cogMap,finalParseState,room,object):
    cogMap.fsa.stateTurnsOnState(parseStateCells,finalParseState,
                                 cogMap.placeBindOnCells,room)
    cogMap.fsa.stateTurnsOnState(parseStateCells,finalParseState,
                                 cogMap.objectBindOnCells,object)
    cogMap.fsa.stateHalfTurnsOnState(parseStateCells,finalParseState,
                                     cogMap.automatonCells,9)#1 startUnbindState
    cogMap.fsa.stateTurnsOffState(cogMap.automatonCells,9,parseStateCells,
                                  finalParseState)#4

    #turn off placeBindOn and objectBindOn cells
    cogMap.fsa.oneNeuronTurnsOffState(cogMap.unbindTimer.timerCells,2569,
                                      cogMap.placeBindOnCells,0)
    cogMap.fsa.oneNeuronTurnsOffState(cogMap.unbindTimer.timerCells,2569,
                                      cogMap.placeBindOnCells,1)
    cogMap.fsa.oneNeuronTurnsOffState(cogMap.unbindTimer.timerCells,2569,
                                      cogMap.placeBindOnCells,2)
    cogMap.fsa.oneNeuronTurnsOffState(cogMap.unbindTimer.timerCells,2569,
                                      cogMap.placeBindOnCells,3)
    cogMap.fsa.oneNeuronTurnsOffState(cogMap.unbindTimer.timerCells,2569,
                                      cogMap.objectBindOnCells,0)
    cogMap.fsa.oneNeuronTurnsOffState(cogMap.unbindTimer.timerCells,2569,
                                      cogMap.objectBindOnCells,1)
    
    

def connectUnbind (parser,cogMap):
    #undone
    aU1FinalState = 65 #apple unbind from room 1
    aU2FinalState = 67
    aU3FinalState = 69
    aU4FinalState = 71
    bU1FinalState = 76 
    bU2FinalState = 78
    bU3FinalState = 80 
    bU4FinalState = 82 

    parseStateCells = parser.getStateCells()
    connectAnUnbindPair(parseStateCells,cogMap,aU1FinalState,1,0)
    
    room = 0
    #And pick the object.
    object = 0


def connectQuerys (parser,cogMap):
    wAFinalState = 15
    wBFinalState = 17
    w1FinalState = 28
    w2FinalState = 30
    w3FinalState = 32
    w4FinalState = 34

    parseStateCells = parser.getStateCells()
    
    #This is asking where is the 0 object?
    #Where is the apple? 0 object
    cogMap.stateTurnsOnObjectQuery(parseStateCells,wAFinalState,0)
    cogMap.objectQueryAnswerTurnsOffState(parseStateCells,wAFinalState)

    #Where is the banana?
    cogMap.stateTurnsOnObjectQuery(parseStateCells,wBFinalState,1)
    cogMap.objectQueryAnswerTurnsOffState(parseStateCells,wBFinalState)

    #What is in room 1?
    cogMap.stateTurnsOnPlaceQuery(parseStateCells,w1FinalState,0)
    cogMap.placeQueryAnswerTurnsOffState(parseStateCells,w1FinalState)

    #What is in room 2?
    cogMap.stateTurnsOnPlaceQuery(parseStateCells,w2FinalState,1)
    cogMap.placeQueryAnswerTurnsOffState(parseStateCells,w2FinalState)

    #What is in room 3?
    cogMap.stateTurnsOnPlaceQuery(parseStateCells,w3FinalState,2)
    cogMap.placeQueryAnswerTurnsOffState(parseStateCells,w3FinalState)

    #What is in room 4?
    cogMap.stateTurnsOnPlaceQuery(parseStateCells,w4FinalState,3)
    cogMap.placeQueryAnswerTurnsOffState(parseStateCells,w4FinalState)
    #In this version of the bind test, pick the room (you can't retrieve 4)
    
def connectParserAndCogMap(parser,cogMap):
    connectQuerys(parser,cogMap)
    #do the connectBind sentences to the cogmap
    connectBind(parser,cogMap)
    #do the connectUnbind from here when you have the final parser states
    connectUnbind(parser,cogMap)


def connectCogMapAndGenerator(cogMap,generator):
    global SentenceFSAs
    objectAnswerCells = cogMap.getObjectAnswerCells()
    placeAnswerCells = cogMap.getPlaceAnswerCells()
    automatonCells = cogMap.getAutomatonCells()

    #The apple is there.
    fsa.stateTurnsOnState(objectAnswerCells,0,SentenceFSAs,0)
    #The banana is there.
    fsa.stateTurnsOnState(objectAnswerCells,1,SentenceFSAs,5)
    #It is in room 1.
    fsa.stateTurnsOnState(placeAnswerCells,0,SentenceFSAs,10) 
    #It is in room 2.
    fsa.stateTurnsOnState(placeAnswerCells,1,SentenceFSAs,16) 
    #It is in room 3.
    fsa.stateTurnsOnState(placeAnswerCells,2,SentenceFSAs,22) 
    #It is in room 3.
    fsa.stateTurnsOnState(placeAnswerCells,3,SentenceFSAs,28)

    #Hook up the nothing and nowhere responses.
    fsa.stateTurnsOnState(automatonCells,17,SentenceFSAs,34)
    fsa.stateTurnsOnState(automatonCells,18,SentenceFSAs,41)
