""" 
This is the nest agent for cabot4
It's paired with an environment that is only the agents orientation 0,90,180 
or 270 and position x,y.  We're in the midst of naming things in places and
using the cogmap.
"""

import sys
import time

import pyNN.nest as sim

from nealCoverClass import NealCoverFunctions
from stateMachineClass import FSAHelperFunctions
from parseClass import ParseClass
import cabot4InputLang as languageToDefine
from cABot4PlanClass import PlanClass
from cogmapClass import cogmapBaseClass
import generatorFile as generator
import CABot4Agent as CABot4Agent

numberPlanActions = -1;

global WordPop

def trySendingWords(words,sendTime):
    for wordNum in range (0,len(words)):
        aWordSpike = words[wordNum]
        spikes = aWordSpike[1]
        aWord = aWordSpike[0]
        for aSpike in spikes:
            if (aSpike == sendTime):
                action = aWord+numberPlanActions #actions first then gen words
                #Actually send it
                setNestOutput(sendTime,action)
                print ('newWordOutput', sendTime,action)


def setNestGeneratorOutput(currentSimulationTime):
    global WordPop
    words = newWordOutput(currentSimulationTime,WordPop)
    #Send words in the order they came
    for sendTime in range (currentSimulationTime-30,currentSimulationTime):
        trySendingWords(words,sendTime)

def newWordOutput(currentSimulationTime,outputWordCells):
    output = []
    actionData = outputWordCells.get_data()
    actionSegments = actionData.segments[0]
    actionSpikeTrains = actionSegments.spiketrains
    #consider this seems to whack on some extra times 
    for action in range (0,len(actionSpikeTrains)):
        if (len(actionSpikeTrains[action]) > 0):
            actionSpikes = actionSpikeTrains[action]
            output = output + [[action,actionSpikes]]
            

    return output

#-----

def newActionOutput(currentSimulationTime,actionCells):
    output = []
    actionData = actionCells.get_data()
    actionSegments = actionData.segments[0]
    actionSpikeTrains = actionSegments.spiketrains
    for action in range (0,len(actionSpikeTrains)):
        if (len(actionSpikeTrains[action]) > 0):
            anActionsSpikes = actionSpikeTrains[action]
            for spike in range (0,len(anActionsSpikes)):
                aSpike = anActionsSpikes[spike]
                if (aSpike <= currentSimulationTime) and (aSpike > currentSimulationTime-30):
                    output = output + [action]
                    print ('newActionOutput', currentSimulationTime,action)

    return output

    
#If haven't started running yet, open the actions file.
def openNestOutput(currentSimulationTime):
    if (currentSimulationTime == 0):
        nestOutputFileHandle = open("actions.txt",'a')
        nestOutputFileHandle.write(str(currentSimulationTime)+"\n")
        nestOutputFileHandle.close()
        return True
    else:
        return False
    
def setNestPlanOutput(currentSimulationTime,actionCells):
    if (openNestOutput(currentSimulationTime)):
        return
    
    actions = newActionOutput(currentSimulationTime,actionCells) 
    for actionNum in range (0,len(actions)):
        setNestOutput(currentSimulationTime,actions[actionNum])
        

def setNestOutput(currentSimulationTime,actionValue):
        nestOutputFileHandle = open("actions.txt",'a')
        nestOutputFileHandle.write(str(actionValue)+" "+
                                   str(currentSimulationTime)+"\n")
        nestOutputFileHandle.close()


def setNestTextOutput(currentSimulationTime):
    global WordPop
    if (currentSimulationTime == 0):
        nestOutputFileHandle = open("outputWords.txt",'a')
        nestOutputFileHandle.write(str(currentSimulationTime)+"\n")
        nestOutputFileHandle.close()
        return
    words = newWordOutput(currentSimulationTime,WordPop) 
    if (len(words) > 0):
        nestOutputFileHandle = open("outputWords.txt",'a')
        for wordNum in range (0,len(words)):
            nestOutputFileHandle.write(str(words[wordNum])+" "+
                                       str(currentSimulationTime)+"\n")
        nestOutputFileHandle.close()


def setNestInput(currentSimulationTime,parser):
    parser.setNestInputs(currentSimulationTime)

    

def runAgent(parser,actionCells):
    callbackTimeStepLength = 30
    steps = int(SIM_LENGTH/callbackTimeStepLength)
    for step in range (0, steps):
        currentTime = step*callbackTimeStepLength
        setNestInput(currentTime,parser)
        setNestPlanOutput(currentTime,actionCells)
        if (currentTime > 0):
            setNestGeneratorOutput(currentTime)
        #setNestTextOutput(currentTime)
        print ("nest simulation timestep ", currentTime)
        sim.run(callbackTimeStepLength)
        time.sleep(0.7)


#---main
simName = "nest"
spinnVersion = -1
neal = NealCoverFunctions(simName,sim,spinnVersion)
fsa = FSAHelperFunctions(simName,sim,neal,spinnVersion)

SIM_LENGTH = 1000.0
SIM_LENGTH = 2000.0
SIM_LENGTH = 4000.0
#SIM_LENGTH = 26000.0

CABot4Agent.init(sim,neal,fsa) 

parser = languageToDefine.main(simName,sim,neal,spinnVersion)
planner = PlanClass(simName,sim,neal,spinnVersion,fsa)
planner.connectCompoundPlanner() #call the "main" of the cabot4 planner
plannerActionCells = planner.getActionCells()
numberPlanActions = planner.NUMBER_ACTIONS

cognitiveMap = cogmapBaseClass(simName,sim,neal,spinnVersion,fsa)

CABot4Agent.buildCognitiveMap(cognitiveMap)
cognitiveMap.useUnbinding()


generatorObject = CABot4Agent.setupGenerator()
WordPop = generatorObject[2]

CABot4Agent.connectParserAndPlanner(parser,planner)
CABot4Agent.connectParserAndCogMap(parser,cognitiveMap)
#CABot4Agent.connectParserAndGenerator(parser,generatorObject)
CABot4Agent.connectCogMapAndGenerator(cognitiveMap,generatorObject)

#testPlanCogMap(planner)
#testGenerator(generatorObject)

#CABot4Agent.fillCogMap(cognitiveMap)


neal.nealApplyProjections()
runAgent(parser,plannerActionCells,)

parser.printResults()
planner.printResults()
cognitiveMap.printCogMapNets()

CABot4Agent.printGenResults('results/')
#generator.printResults(generator.SentenceFSAs,'results/genStates.pkl')
#generator.printResults(WordPop,'results/genWords.pkl')
#generator.printResults(PhraseCAs[0],'results/genPhrases.pkl')

sim.end()


