""" 
This is the nest agent for cabot4
It's paired with an environment that is only the agents orientation 0,90,180 
or 270 and position x,y.  We're in the midst of naming things in places and
using the cogmap.
"""

import sys
import time

import pyNN.spiNNaker as sim
import spynnaker8.external_devices as ExternalDevices

from nealCoverClass import NealCoverFunctions
from stateMachineClass import FSAHelperFunctions
from parseClass import ParseClass
import cabot4InputLang as languageToDefine
from cABot4PlanClass import PlanClass
from cogmapClass import cogmapBaseClass
import generatorFile as generator
import CABot4Agent as CABot4Agent

numberPlanActions = -1;

global WordPop

def linkInSink(spikeSource,sinkCell):
    if (nealParameters.simulator == 'spinnaker'):
        connector = [(0,0,1.0, nealParameters.DELAY)]
        neal.nealProjection(spikeSource,sinkCell,connector,'inhibitory') 

#spinnaker crashes if a spike source isn't connected to
#something.  Make the something here.
def allocateSinks(firstSink,generators):
    if (nealParameters.simulator != 'spinnaker'):
        return
    sinkCell=sim.Population(1,sim.IF_cond_exp,{})
    for sinkNumber  in range (firstSink,7):
        linkInSink(spikeGenerators[sinkNumber],sinkCell)


def runAgent():
    sim.run(SIM_LENGTH)



#---main
simName = "spinnaker"
spinnVersion = 8
neal = NealCoverFunctions(simName,sim,spinnVersion)
fsa = FSAHelperFunctions(simName,sim,neal,spinnVersion)

#SIM_LENGTH = 2000.0
#SIM_LENGTH = 6000.0
#SIM_LENGTH = 20000.0
SIM_LENGTH = 30000.0
#SIM_LENGTH = 40000.0

CABot4Agent.init(sim,neal,fsa)

parser = languageToDefine.main(simName,sim,neal,spinnVersion)
parser.allocateSpinnakerInputs()
parser.connectSpinnakerInputsToWords()
planner = PlanClass(simName,sim,neal,spinnVersion,fsa)
planner.connectCompoundPlanner() #call the "main" of the cabot4 planner
plannerActionCells = planner.getActionCells()
numberPlanActions = planner.NUMBER_ACTIONS

cognitiveMap = cogmapBaseClass(simName,sim,neal,spinnVersion,fsa)

CABot4Agent.buildCognitiveMap(cognitiveMap)
cognitiveMap.useUnbinding()


generatorObject = CABot4Agent.setupGenerator()
WordPop = generatorObject[2]
ExternalDevices.activate_live_output_for(WordPop,
                    database_notify_port_num=12346)


CABot4Agent.connectParserAndPlanner(parser,planner)
CABot4Agent.connectParserAndCogMap(parser,cognitiveMap)
#CABot4Agent.connectParserAndGenerator(parser,generatorObject)
CABot4Agent.connectCogMapAndGenerator(cognitiveMap,generatorObject)


#testPlanCogMap(planner)
#testGenerator(generatorObject)

#CABot4Agent.fillCogMap(cognitiveMap)

neal.nealApplyProjections()
runAgent()

#dumpSpikes = False
#dumpSpikes = True
#if (dumpSpikes):
parser.printResults()
planner.printResults()
cognitiveMap.printCogMapNets()
CABot4Agent.printGenResults('results/')

#generator.printResults(generator.SentenceFSAs,'results/genStates.pkl')
#generator.printResults(WordPop,'results/genWords.pkl')
#generator.printResults(PhraseCAs[0],'results/genPhrases.pkl')

sim.end()


