"""
This plan class has four primitive goal-module-action triples,
and two compounds with a timer.  """

from planBaseClass import PlanBaseClass
from timerClass import TimerClass

class PlanClass (PlanBaseClass):
    NUMBER_GOALS = 13
    goalTurnRight = 0
    goalTurnLeft = 1
    goalStepForward = 2
    goalStepBackward = 3
    goalForwardAndRight = 4
    goalForwardAndLeft = 5
    #undone remove these
    goalQueryYB = 9 #where is the YB
    goalQueryRA = 10
    goalQuery1 = 11  #what's in 1,1
    goalQuery2 = 12


    NUMBER_MODULES = 15
    moduleTurnRight = 0
    moduleTurnLeft = 1
    moduleStepForward = 2
    moduleStepBackward = 3
    moduleForwardOfForwardRight = 4
    moduleRightOfForwardRight = 5
    moduleForwardOfForwardLeft = 6
    moduleLeftOfForwardLeft = 7
    #undone remove these extra modules
    modulePutYBAt1 = 8
    modulePutYBAt2 = 9 
    modulePutRAAt1 = 10
    moduleQueryYB = 11
    moduleQueryRA = 12
    moduleQuery1 = 13
    moduleQuery2 = 14

    NUMBER_FACTS = 1
    bumpFact = 0

    NUMBER_ACTIONS = 15
    actionRight = 0
    actionLeft = 1
    actionForward = 2
    actionBackward = 3
    #undone remove these extra ones.
    actionPutYBAt1 = 4
    actionPutYBAt2 = 5 
    actionPutRAAt1 = 6
    actionQueryYB = 7
    actionQueryRA = 8
    actionQuery1 = 9
    actionQuery2 = 10
    actionAnswer1 = 11
    actionAnswer2 = 12
    actionAnswerYB = 13
    actionAnswerRA = 14
    #Consider can add answers nothing, nowhere, and don't know via
    #the fail answers in cogmap.

    def __init__(self, simName,sim,neal,spinnVersion,fsa):
        PlanBaseClass.__init__(self,simName,sim,neal,spinnVersion,fsa)
        self.simName = simName
        self.sim = sim
        self.neal = neal
        self.spinnVersion = spinnVersion
        self.fsa = fsa
        self.initializeBasePlanner()

    def makeTimers(self):
        self.forwardRightTimer = TimerClass(self.simName,self.sim,self.neal,
                                            self.spinnVersion,self.fsa,5)

        self.forwardLeftTimer = TimerClass(self.simName,self.sim,self.neal,
                                            self.spinnVersion,self.fsa,5)

    def connectCompoundPlanner(self):
        self.makeTimers()
        self.makePrimitive(self.goalTurnRight,self.moduleTurnRight,
                           self.actionRight)
        self.makePrimitive(self.goalTurnLeft,self.moduleTurnLeft,
                           self.actionLeft)
        self.makePrimitive(self.goalStepForward,self.moduleStepForward,
                           self.actionForward)
        self.makePrimitive(self.goalStepBackward,self.moduleStepBackward,
                           self.actionBackward)

        self.forwardRightTimer.makeStopTimerSynapses()
        self.makeCompound(self.goalForwardAndRight,
                          self.moduleForwardOfForwardRight,
                          self.moduleRightOfForwardRight,self.actionForward,
                          self.actionRight,self.forwardRightTimer)

        self.forwardLeftTimer.makeStopTimerSynapses()
        self.makeCompound(self.goalForwardAndLeft,
                          self.moduleForwardOfForwardLeft,
                          self.moduleLeftOfForwardLeft,self.actionForward,
                          self.actionLeft,self.forwardLeftTimer)
 
    def printResults(self):
        PlanBaseClass.printResults(self)
        self.forwardRightTimer.printResults("results/planForwardRightTimer")
        self.forwardLeftTimer.printResults("results/planForwardLeftTimer")

    def testStimulateFact(self,factNumber,spikeGenerator):
        self.fsa.turnOnStateFromSpikeSource(spikeGenerator,
                                            self.factCells, factNumber)

