""" 
Using the parseClass, build a topology that speficies a parser to parse
these sentences.  

Turn left.
Turn right.
Step forward.
Step back.
Move left.
Move right.
Where is the apple?
Where is the banana?
What is in room 1?
What is in room 2?
What is in room 3?
What is in room 4?
"""

import sys

from parseClass import ParseClass

totalSentences = -1#set explicitly
lengthOfLongestSentence = -1 #set explicitly
neal = None
sim = None
simName = None
spinnVersion = None

def addTurnSentences(parser):
    parser.addSentence(['turn','left','.'])
    turnState = parser.addStateToNewStateOnWord(parser.startState,'turn')
    turnLeftState = parser.addStateToNewStateOnWord(turnState,'left')
    tLPFinalState = parser.addStateToNewStateOnWord(turnLeftState,'.')

    parser.addSentence(['turn','right','.'])
    turnRightState = parser.addStateToNewStateOnWord(turnState,'right')
    tRightFinalState = parser.addStateToNewStateOnWord(turnRightState,'.')

    return [tLPFinalState,tRightFinalState]


def addStepSentences(parser):
    #specify the step forward sentence.
    parser.addSentence(['step','forward','.']) 
    stepState = parser.addStateToNewStateOnWord(parser.startState,'step')
    stepForwardState = parser.addStateToNewStateOnWord(stepState,'forward')
    stepForwardFinalState = parser.addStateToNewStateOnWord(stepForwardState,'.')

    #specify the step forward sentence.
    parser.addSentence(['step','backward','.']) 
    stepBackwardState = parser.addStateToNewStateOnWord(stepState,'backward')
    stepBackwardFinalState = parser.addStateToNewStateOnWord(stepBackwardState,'.')

    return [stepForwardFinalState,stepBackwardFinalState]
    

def addWhereSentences(parser):
    #specify the where is the apple ?
    parser.addSentence(['where','is','the','apple','?']) 
    whereState = parser.addStateToNewStateOnWord(parser.startState,'where')
    whereIsState = parser.addStateToNewStateOnWord(whereState,'is')
    whereIsTheState = parser.addStateToNewStateOnWord(whereIsState,'the')
    whereIsTheAppleState = parser.addStateToNewStateOnWord(whereIsTheState,
                                                           'apple')
    appleFinalState = parser.addStateToNewStateOnWord(whereIsTheAppleState,
                                                      '?')

    #specify the where is the banana ?
    parser.addSentence(['where','is','the','banana','?']) 
    whereIsTheBananaState = parser.addStateToNewStateOnWord(whereIsTheState,
                                                           'banana')
    bananaFinalState = parser.addStateToNewStateOnWord(whereIsTheBananaState,
                                                      '?')

    return [appleFinalState,bananaFinalState]

def addMoveSentences(parser):
    parser.addSentence(['move','left','.'])
    moveState = parser.addStateToNewStateOnWord(parser.startState,'move')
    moveLeftState = parser.addStateToNewStateOnWord(moveState,'left')
    mLFinalState = parser.addStateToNewStateOnWord(moveLeftState,'.')

    parser.addSentence(['move','right','.'])
    moveRightState = parser.addStateToNewStateOnWord(moveState,'right')
    mRFinalState = parser.addStateToNewStateOnWord(moveRightState,'.')

    return [mLFinalState,mRFinalState]
    
def addWhatSentences(parser):
    #specify the what is in room one ?
    parser.addSentence(['what','is','in','room','1','?']) 
    whatState = parser.addStateToNewStateOnWord(parser.startState,'what')
    whatIsState = parser.addStateToNewStateOnWord(whatState,'is')
    whatIsInState = parser.addStateToNewStateOnWord(whatIsState,'in')
    whatIsInRoomState = parser.addStateToNewStateOnWord(whatIsInState,'room')
    whatIsInRoom1State = parser.addStateToNewStateOnWord(whatIsInRoomState,'1')
    room1FinalState = parser.addStateToNewStateOnWord(whatIsInRoom1State, '?')

    parser.addSentence(['what','is','in','room','2','?']) 
    whatIsInRoom2State = parser.addStateToNewStateOnWord(whatIsInRoomState,'2')
    room2FinalState = parser.addStateToNewStateOnWord(whatIsInRoom2State, '?')
    
    parser.addSentence(['what','is','in','room','3','?']) 
    whatIsInRoom3State = parser.addStateToNewStateOnWord(whatIsInRoomState,'3')
    room3FinalState = parser.addStateToNewStateOnWord(whatIsInRoom3State, '?')
    
    parser.addSentence(['what','is','in','room','4','?']) 
    whatIsInRoom4State = parser.addStateToNewStateOnWord(whatIsInRoomState,'4')
    room4FinalState = parser.addStateToNewStateOnWord(whatIsInRoom4State, '?')
    
    return[room1FinalState,room2FinalState,room3FinalState,room4FinalState]

def linkInSink(spikeSource,sinkCell):
    global simName
    global neal
    if (simName == 'spinnaker'):
        connector = [(0,0,1.0, neal.DELAY)]
        neal.nealProjection(spikeSource,sinkCell,connector,'inhibitory') 


#spinnaker crashes if a spike source isn't connected to
#something.  Make the something here.
def allocateSinks(firstSink,spikeGenerators):
    global simName
    global sim
    if (simName != 'spinnaker'):
        return
    sinkCell=sim.Population(1,sim.IF_cond_exp,{})
    for sinkNumber  in range (firstSink,7):
        linkInSink(spikeGenerators[sinkNumber],sinkCell)

def setupTestSentences(parser,spikeGenerators,sentence):
    if (sentence == 0):
        parser.testTurnOnWord("turn",spikeGenerators[1])
        parser.testTurnOnWord("left",spikeGenerators[2])
        parser.testTurnOnWord(".",spikeGenerators[3])
        allocateSinks(4,spikeGenerators)
    elif (sentence == 1):
        parser.testTurnOnWord("step",spikeGenerators[1])
        parser.testTurnOnWord("forward",spikeGenerators[2])
        parser.testTurnOnWord(".",spikeGenerators[3])
        allocateSinks(4,spikeGenerators)


#access functions for variables from this file
def getTotalSentences():
    return totalSentences

def getLengthOfLongestSentence():
    return lengthOfLongestSentence

#---main 
def main(simNameInput,simInput,nealInput,spinnVersionInput):
    global neal
    global sim
    global simName
    global spinnVersion

    global totalSentences
    global lengthOfLongestSentence

    #allocate neal and a parser
    neal = nealInput
    sim = simInput
    simName = simNameInput
    spinnVersion = spinnVersionInput
    parser = ParseClass(simName,sim,neal,spinnVersion)

    #specify the sentences you want to parse.
    tFinalStates = addTurnSentences(parser)
    tFinalStates = tFinalStates + addStepSentences(parser)
    tFinalStates = tFinalStates + addWhereSentences(parser)
    tFinalStates = tFinalStates + addMoveSentences(parser)
    tFinalStates = tFinalStates + addWhatSentences(parser)

    totalSentences = 12
    lengthOfLongestSentence = 6 #0 based

    #optionally, print out information about the parser
    parser.printWords()
    #parser.printStateTransitions()
    #parser.printWordTransitions()

    print ("Final states in order")
    print (tFinalStates)
    
    #allocate the topology
    parser.createNeurons()
    parser.recordNeurons()
    parser.createCAs()
    parser.createTransitionSynapses()

    #access the topology via the parser
    return parser



