"""
This is the base version of the world.  It's got a text box that you can
put inputs into.  These are then sent to the agent in the form of 
spikes, it processes them, and returns an action.  The communication is
done differently between spinnaker and nest, so the class needs 
to be included.
"""

from tkinter import *

import math
#import time

FIRST_WORD_COMMAND = 15

class Environment(Frame):
    direction = 0.0
    locationX = 0.0
    locationY = 0.0

    numFiles = 0
    numUserFiles = 0

    #iOObject deals with io for nest or spinnaker depending
    def __init__(self, parent,fNest,inputOutputObject):  
        self.fileNum = 0
        Frame.__init__(self, parent)   
        self.iOObject = inputOutputObject
        
        #set up call back to interact with agent
        if (fNest):
            self.after(200,self.getNestInput)
        else:
            self.after(200,self.getSpinnInputOutput)

        self.parent = parent        
        self.initUI()

    def getNestInput(self):
        action = self.iOObject.getAction()
        if (action >= 0):
            self.processCommandFromBoard(action)
        
        self.iOObject.increaseTimeTicks()
        self.after(200,self.getNestInput)
        
    def getSpinnInputOutput(self):
        if (self.iOObject.fHaveCommandFromBoard):
            self.processCommandFromBoard(self.iOObject.getCommandFromBoard())
        self.iOObject.sendWord()
        self.after(200,self.getSpinnInputOutput)


    def processTextInput (self, textInput):
        #clear whitespace before and after
        input = textInput.rstrip()
        input = input.lstrip()

        #test for characters.
        if input == "l": 
            self.iOObject.writeSentence([-1,0,1,2],4)
        elif input == "r":
            self.iOObject.writeSentence([-1,0,3,2],4)
        elif input == "f":
            self.iOObject.writeSentence([-1,4,5,2],4)
        elif input == "step forward.":
            self.iOObject.writeSentence([-1,4,5,2],4)
        elif input == "b":
            self.iOObject.writeSentence([-1,4,6,2],4)
        elif input == "w":
            self.iOObject.writeSentence([-1,7,8,9,10,11],6)
        elif input == "where is the apple?":
            self.iOObject.writeSentence([-1,7,8,9,10,11],6)
        elif input == "y":
            self.iOObject.writeSentence([-1,7,8,9,12,11],6)
        elif input == "where is the banana?":
            self.iOObject.writeSentence([-1,7,8,9,12,11],6)
        elif (input == "L") or (input == "move left."):
            self.iOObject.writeSentence([-1,13,1,2],4)
        elif (input == "R") or (input == "move right."):
            self.iOObject.writeSentence([-1,13,3,2],4)
        elif input == "1":
            self.iOObject.writeSentence([-1,14,8,15,16,17,11],7)
        elif input == "what is in room 1?":
            self.iOObject.writeSentence([-1,14,8,15,16,17,11],7)
        elif input == "2":
            self.iOObject.writeSentence([-1,14,8,15,16,18,11],7)
        elif input == "what is in room 2?":
            self.iOObject.writeSentence([-1,14,8,15,16,18,11],7)
        elif input == "3":
            self.iOObject.writeSentence([-1,14,8,15,16,19,11],7)
        elif input == "4":
            self.iOObject.writeSentence([-1,14,8,15,16,20,11],7)
        else:
            print ("bad text input ", textInput)


    fPrintedCommand = False
    fPrintedNoCommand = False

    #consider need to link this to the words from the generator
    def getGenWords(self,commandNum):
        if (commandNum == 15):
            return "The"
        elif (commandNum == 16):
            return "apple"
        elif (commandNum == 17):
            return "is"
        elif (commandNum == 18):
            return "there"
        elif (commandNum == 19):
            return "banana"
        elif (commandNum == 20):
            return "It"
        elif (commandNum == 21):
            return "in"
        elif (commandNum == 22):
            return "room"
        elif (commandNum == 23):
            return "1"
        elif (commandNum == 24):
            return "2"
        elif (commandNum == 25):
            return "3"
        elif (commandNum == 26):
            return "4"
        elif (commandNum == 28):
            return "."
        else:
            print ("error bad command to genWords ", commandNum)

    def addInputFromGenerator(self,commandNum):
        if (commandNum == FIRST_WORD_COMMAND):
            self.editBox.delete(1.0,END)
        textString = self.getGenWords(commandNum)
        self.editBox.insert(END,textString)
        self.editBox.insert(END," ")

    def processCommandFromBoard(self,commandNum):
        if (commandNum == 0):
            self.turnLeft()
        elif (commandNum == 1):
            self.turnRight()
        elif (commandNum == 2):
            self.moveForward()
        elif (commandNum == 3):
            self.moveBack()
        elif (commandNum >= FIRST_WORD_COMMAND):
            self.addInputFromGenerator(commandNum)
            print ("word" + str (commandNum))            
        else:
            print ("environment ignores action ", commandNum)


    #-----Functions for moving
    def printPosition(self):
        print (self.direction,self.locationX,self.locationY)

    def moveForward(self):
        print ("Move Forward")
        if self.direction == 0.0:
            self.locationX = self.locationX + 1.0
        elif self.direction == 90.0:
            self.locationY = self.locationY + 1.0
        elif self.direction == 180.0:
            self.locationX = self.locationX - 1.0
        elif self.direction == 270.0:
            self.locationY = self.locationY - 1.0
        self.printPosition()

    def moveBack(self):
        if self.direction == 0.0:
            self.locationX = self.locationX - 1.0
        elif self.direction == 90.0:
            self.locationY = self.locationY - 1.0
        elif self.direction == 180.0:
            self.locationX = self.locationX + 1.0
        elif self.direction == 270.0:
            self.locationY = self.locationY + 1.0
        self.printPosition()

    def turnLeft (self):
        self.direction = self.direction - 90.0
        self.direction = (self.direction % 360.0)
        self.printPosition()

    def turnRight (self):
        self.direction = self.direction + 90.0
        self.direction = (self.direction % 360.0)
        self.printPosition()

    #send the sentence when you hit return
    def keyHandler(self,event):
        #print "keyhandler", repr(event.char)
        if (event.char == '\r'):
            text = self.editBox.get(1.0,END)
            self.editBox.delete(1.0,END)
            self.processTextInput(text)
            #self.putTextInFile(text)

    #set up the tickle frame with and edit box, buttons
    def initUI(self):
        self.parent.title("Turn Step Environment")        
        self.pack(fill=BOTH, expand=1)

        self.canvas = Canvas(self)

        self.canvas.pack(fill=BOTH, expand=1)

        quitButton = Button(self, text="Quit", command=self.quit)
        quitButton.place(x=0, y=0)
        printButton = Button(self, text="Print", command=self.printPosition)
        printButton.place(x=420, y=0)

        self.editBox = Text(self,height=1, width=50)
        self.editBox.place(x=0, y=25)
        self.editBox.bind("<Key>",self.keyHandler)
    #end of class





