from nealCoverClass import NealCoverFunctions
import re
#import cPickle as pickle
import numpy as np
import itertools as combi
from stateMachineClass import FSAHelperFunctions

#---Below here are functions to test the system---
RUN_DURATION = 200#200
#initialize the simulator.

WORD_CELL_PARAMS = {'v_thresh':-48.0, 'v_reset' : -70.0, 
                                'tau_refrac': 45.0 , 'tau_syn_E': 5.0,  
                                'tau_syn_I': 5.0, #bug2 fix
                                'v_rest' : -65.0,'i_offset':0.0}

def init(neal):
    neal.sim.setup(timestep=neal.DELAY,min_delay=neal.DELAY,
                    max_delay=neal.DELAY, debug=0)

def createOneInput(fsa):
    inputSpikeTimes0 = [10.0]
    spikeArray0 = {'spike_times': [inputSpikeTimes0]}
    spikeGen0=fsa.sim.Population(1,fsa.sim.SpikeSourceArray,spikeArray0,
                                   label='inputSpikes_0')    

    return [spikeGen0]

def readFile(filename):
    Senfile = open(filename,"r") #opens sentence file
    AllLines = []
    for eachline in Senfile:
        AllLines.append(eachline.strip()) #strip gets rid of /n

    #print(AllLines)

    Senfile.close()
    return AllLines
    
def createTwoInputs():
    inputSpikeTimes0 = [10.0]
    inputSpikeTimes1 = [50.0]
    spikeArray0 = {'spike_times': [inputSpikeTimes0]}
    spikeGen0=sim.Population(1,sim.SpikeSourceArray,spikeArray0,
                                   label='inputSpikes_0')
    spikeArray1 = {'spike_times': [inputSpikeTimes1]}
    spikeGen1=sim.Population(1, sim.SpikeSourceArray, spikeArray1,
                                   label='inputSpikes_1')

    return [spikeGen0,spikeGen1]

def createNeurons(NumAssemblies,fsa):
    numNeurons = fsa.CA_SIZE * NumAssemblies
    cells=fsa.sim.Population(numNeurons,fsa.sim.IF_cond_exp,fsa.CELL_PARAMS)
    print ("generator state neurons " + str (numNeurons))
    return cells

def createWordNeurons(fsa,NumAssemblies,CA_SIZE):
    #we want a long refractory period (45ms) to stop the same word repeatedly firing, this is due to the half on state connections,
    #as two half state inputs take a while to fire the next state, every time a CA fire, it triggers the same word neuron. Therefore, we
    #want to wait, i.e. use a long refractory period, until the next state fires to inhibit the word.
    numNeurons = CA_SIZE * NumAssemblies

    if (fsa.simName == 'spinnaker')and(fsa.spinnVersion == 8) :
        print ("spinnaker 8 for connecting words to board")
        import spynnaker8.external_devices as ExternalDevices
        cells=fsa.neal.sim.Population(numNeurons,
                    fsa.neal.sim.IF_cond_exp, WORD_CELL_PARAMS,
                    label='wordFromBoard')
        ExternalDevices.activate_live_output_for(cells,
                    database_notify_port_num=12346)
    else: #nest
        cells=fsa.neal.sim.Population(numNeurons,fsa.neal.sim.IF_cond_exp,WORD_CELL_PARAMS)
        print ("generator words " + str(numNeurons))

    return cells

def setupRecording(sentenceCells,wordCells,phraseCells):
    sentenceCells.record({'spikes','v'})
    wordCells.record({'spikes','v'})
    phraseCells.record({'spikes','v'})


def printResults(sentenceCells,wordCells,parseCells,fileName):
    sentenceCells.write_data(fileName+'/genSentence.pkl')
    wordCells.write_data(fileName+'/genWords.pkl')
    parseCells.write_data(fileName+'/genPhrase.pkl')

def ExtractSentencesAndPhrases(AllLines):
    ##extract the sentences and phrases from a file
    ##return 2 lists, first list (StoredSenAndPhrases)  contains a set of lists, where the first list contains the sentence templates and the
    ##other lists contain phrase sets.  Each phrase list represents one variable, and each list contains the variable name.  we add the "NONE"
    #phrase to each phrase list which is used when no word/phrase is required
    ##the second list (NumPhrasesList) contains the number of sentences (including the sentence header) and phrases associated to each variable
    ##(containing the variable name)

    ##Number Of Sentences
    NumLines=len(AllLines)

    StoredSenAndPhrases=[[]]
    TotalNumVars=0
    NumPhrasesList=[]
    NumPhrases=0
    totalNumSentences=1
    for eachLine in range(0,NumLines):
        Words=AllLines[eachLine].replace('.',' .') ##remove . from sentence ##DO THIS
        Words=Words.split()
        #print "words"
        #print Words
        
        NumWords=len(Words)
        if((NumWords==1)and("?" in AllLines[eachLine])):  #IF A VARIABLE
            for VarLine in range(eachLine,NumLines):
                if(AllLines[VarLine]!=""):
                    StoredSenAndPhrases[TotalNumVars]+=[AllLines[VarLine].replace('.',' .')]
                    NumPhrases+=1
                else:
                    if(TotalNumVars>0):  ##do not add a NONE phrase to sentences list
                        StoredSenAndPhrases[TotalNumVars]+=["NONE"] ##add NONE to phrase list, this will be used if no word/phrase is required
                        NumPhrases+=1
                    StoredSenAndPhrases+=[[]] 
                        
                    NumPhrasesList+=[NumPhrases] #number of phrases associated to each variable (including the variable heading)
                    TotalNumVars+=1
                    #print "numphrase"
                    #print NumPhrases-1
                    #print NumPhrases
                    NumPhrases=0
                    break

    StoredSenAndPhrases.remove([]) #remove the extra []

    #print StoredSenAndPhrases
    #print NumPhrases  ##also includes the word NONE
    return [StoredSenAndPhrases,NumPhrasesList]
            

def CreateWordPop(fsa,AllLines):
    ##create a population of neurons, where each neuron represents a different word

    ##Number Of Sentences
    NumLines=len(AllLines)

    ##find all unique words from all sentences and phrases
    StoredWords=[]
    NumStoredWords=0
    for eachLine in range(0,NumLines):
        Words=AllLines[eachLine].replace('.','') ##remove . from sentence
        Words=Words.split() #split the sentence into words
        
        NumWords=len(Words)
        for eachWord in range(0,NumWords):
            alreadyrecorded=0
            if ("?" not in Words[eachWord]):
                for eachStoredWord in range(0,NumStoredWords):
                
                    if(Words[eachWord]==StoredWords[eachStoredWord]): #if a word has been found before
                        alreadyrecorded=1
                        break

                if(alreadyrecorded==0): #if a word has not been found before, store it and increment population size counter
                    StoredWords+=[Words[eachWord]]
                    NumStoredWords+=1

    StoredWords+=["."] ##add . to the list

    ##create a CA for each word
    numInCellCA=1  #one neuron corrsponds to one word
    WordPop = createWordNeurons(fsa,NumStoredWords+1,numInCellCA) #+1 for full stop

    #print StoredWords

    return [WordPop,StoredWords]

def findWordNeuron(WordsInPop,word):
    ##return the neuron (index) in the word population that represents the given word
    NumberCAWords=len(WordsInPop)
    wordNeuron=[]
    for eachWord in range(0,NumberCAWords):
        #print word
        #print WordInCAs[eachWord]
        if(word==WordsInPop[eachWord]):
            wordNeuron=eachWord
            break

    if(wordNeuron==[]):
        print ("Word not found")
        print (word)
        error

    return wordNeuron

def findStateCA(PhraseSenIndex,StartStateCA,nextword):
    ##return the state (CAindex) associated to the given word or initial state of a phrase in a sentence
    #StartStateCA is the initial (index) state of the sentence
    #print "findStateCA"
    StateIndex=StartStateCA
    #print "PhraseSenIndex"
    #print PhraseSenIndex
    #print "nextword"
    #print nextword
    for i in range(0,len(PhraseSenIndex)):
        if(PhraseSenIndex[i]==" "):
            StateIndex+=1   
        if(PhraseSenIndex[i]!=nextword):
            #print "index"
            #print i
            StateIndex+=len(PhraseSenIndex[i].split()) ##number of states for a phrase equals the number of words in phrase
            #print PhraseSenIndex[i]
        else:
            break

    #print "StateIndex"
    #print StateIndex
    return StateIndex

def findPhraseCA(PhraseCAList,phrase):
    ##find the CA corresponding to the phrase in the phrase CA population
    #print "findStateCA"
    #print "PhraseCAList"
    #print PhraseCAList
    #print "phrase"
    #print phrase
    StateIndex=0
    foundphrase=0
    for i in range(0,len(PhraseCAList)):
        
        if(PhraseCAList[i]==phrase):
            foundphrase=1
            #print "index"
            #print i
            break
        StateIndex+=1

    #print "StateIndex"
    #print StateIndex
    if(foundphrase==0):
        print ("ERROR: cannot find phrase in phrase list:")
        print (phrase)
    return StateIndex

def turnOnStatesAndWordsInPhrases(NumPhraseInfo,StoredSenAndPhrases,nextword,StartStateCA,PhraseSenIndex,State,WordPopInfo,PhraseCAsInfo,prevword,FSASentencesPop):
    #this turns on the next state, i.e. a phrase (words and states) within a sentence
    #the current state (State (word or phrase)) connects to the next state (initial phrase states corresponding to the nextword variable)
    #the initial states trigger the states and words comprising the phrase
    #the initial states turn off the previous final states (if a phrase) or state (if a word)
    ##returns the last word and last states of each phrase

    

    WordPop=WordPopInfo[0]      ##this contains the word population
    Words=WordPopInfo[1]        ##this contains the words each neuron represents

    NumPhraseVars=NumPhraseInfo[0] #number of phrase sets
    NumPhrases=NumPhraseInfo[1] #number of phrases in each set

    PhraseCAs=PhraseCAsInfo[0]              ##Phrase CA population
    PhraseCAIndex=PhraseCAsInfo[1]          ##index of each phrase in phrase CA pop    

    #PhraseSenIndex contains the sentence and all of the phrases that can comprise the sentence.  This is used to determine
                          #the state index of each word and phrase in a sentence
    

    prevstates=[]
    phrases=StoredSenAndPhrases[1:] #extract out all phrases

    #print phrases
    #print NumPhrases
    #print NumPhraseVars

    for eachPhraseSet in range(0,NumPhraseVars):
        PhraseSet=phrases[eachPhraseSet] #phrase set for a particular variable
        #print "nextword"
        #print nextword
        if(nextword==PhraseSet[0]):  ##find the phrase set corresponding to the variable (nextword) given
            for eachPhrase in range(1,NumPhrases[eachPhraseSet]): ##for each phrase (excluding the variable name)
                nextPhrase=PhraseSet[eachPhrase]                                           ##get a phrase
                NextState=findStateCA(PhraseSenIndex,StartStateCA,nextPhrase+PhraseSet[0]) ##find the next state, i.e. the initial state of the phrase, in a given sentence
                                                                                            ##tag the phrase with the associated variable
                                                                                            ##StartStateCA is the index of the initial state of a given sentence
                #print "nextPhrase"
                #print nextPhrase
                #print "NextState"
                #print NextState
                
                ####
                
                if("?" in prevword):  #if the previous word was a variable
                    #print "prevword"
                    #print prevword
                    #print "State"
                    #print State
                    for i in range (0,len(State)): ## connect the final state of all previous phrases to the next state, i.e. the initial state of a phrase
                                                   ##when a previous phrase is triggered, it will half turn on the next state, i.e. the initial state of a phrase
                        fsa.stateHalfTurnsOnState(FSASentencesPop,State[i],FSASentencesPop,NextState)
                        
                else: #if previous state was a word
                    #the previous state (word) will half turn on the next state, i.e. the initial state of a phrase
                    fsa.stateHalfTurnsOnState(FSASentencesPop,State,FSASentencesPop,NextState)
                    
                ####
                PhraseState=findPhraseCA(PhraseCAIndex,nextPhrase+nextword) ##find the CA corresponding to the phrase in the population of phrase CAs
                                                                            ##tag the phrase with the associated variable
                fsa.stateHalfTurnsOnState(PhraseCAs,PhraseState,FSASentencesPop,NextState)
                                                                            ##the phrase that is turned on at the beginning of the sim,
                                                                            ##will half turn on the next state, i.e. the initial state of the phrase
                WordsInPhrase=PhraseSet[eachPhrase].split()
                NumWordsInPhrase=len(WordsInPhrase)
                #print "WordsInPhrase"
                #print WordsInPhrase
                #print "NumWordsInPhrase"
                #print NumWordsInPhrase
                ##turn on the next state and word in a phrase                
                for eachword in range(0,NumWordsInPhrase): 
                    fsa.makeCA(FSASentencesPop,NextState+eachword)      #make a CA for each state in a phrase
                                                                         
                    #print "NextState+eachword"
                    #print NextState+eachword
                    if(eachword+1<NumWordsInPhrase):
                        ##previous phrase state turns on next phrase state
                        fsa.stateTurnsOnState(FSASentencesPop,NextState+eachword,FSASentencesPop,NextState+eachword+1)#the next phrase state (i.e. index) is +1 after the 
                                                                                                                      #after the current phrase state
                                                                                                                      
                                                                                                                      

                    if(WordsInPhrase[eachword]!="NONE"): ## NONE == NO WORD/PHRASE IS REQUIRED.  We do not turn on a word neuron if "NONE" is used                    
                        #turn on the associated word neuron
                        wordNeuron=findWordNeuron(Words,WordsInPhrase[eachword])
                        fsa.stateTurnsOnOneNeuron(FSASentencesPop,NextState+eachword,WordPop,wordNeuron)

                    if(eachword==0):  ##if the current word is the first word in the phrase
                        ##turn off previous state (if a word) or previous final phrase states (if a phrase)
                        ###
                        if("?" in prevword): #if the previous word to the phrase was a variable, connect the initial state of each phrase to the
                                              #previous final state of each phrase
                                             #this will turn off the previous phrase final states when one of the phrases are triggered
                            for i in range (0,len(State)):
                                fsa.stateTurnsOffState(FSASentencesPop,NextState+eachword,FSASentencesPop,State[i])  
                        else:
                            #if the previous word was a word (not a variable) connect the initial state of each phrase to the previous word state
                            fsa.stateTurnsOffState(FSASentencesPop,NextState+eachword,FSASentencesPop,State)
                        ###
                    else:
                        #if current state in phrase is activated, turn off previous state in phrase (-1)
                        fsa.stateTurnsOffState(FSASentencesPop,NextState+eachword,FSASentencesPop,NextState+eachword-1)
    

                    if(eachword==NumWordsInPhrase-1):
                        prevstates+=[NextState+eachword]  #record the last state of each phrase

    #print "prevstates"
    #print prevstates
    return [prevstates,wordNeuron]

def CreateSentenceFSA(fsa,StartStateCA,FSASentencesPop,WordPopInfo,NumberStates,wordsInSen,PhraseCAs,StoredSenAndPhrasesInfo,PhraseSenIndex):
    #create an FSA for one sentence

    StoredSenAndPhrases=StoredSenAndPhrasesInfo[0] ##this contains the sentence templates and the different phrases
    NumPhrases=StoredSenAndPhrasesInfo[1] #this contains the number of phrases (including the variable heading), and sentence templates
                                          #(includng the sentence heading)
    NumSentences=NumPhrases[0] 

    #print NumPhrases
    
    NumPhrases=NumPhrases[1:]  #extract out the number of phrases for each variable
    NumPhraseVars=len(NumPhrases) # number of phrase categories    

    NumPhraseInfo=[NumPhraseVars,NumPhrases]
    
    State=StartStateCA          ##starting state (i.e. index of the initial state in FSASentencesPop) for the given sentence
    WordPop=WordPopInfo[0]      ##this contains the word population
    Words=WordPopInfo[1]        ##this contains the words each neuron represents

    NumWords=len(wordsInSen)
    prevword=[]
    currentWord=wordsInSen[0]

    if("?" in currentWord): ##if first word in the sentence is a variable
        start=-1        
    else:
        start=0
    
    for eachWord in range(start,NumWords): ## for each word (and variable) in the given sentence
        #create a CA for each State (## the states are the caret positions after each
        #word (including the beginning of the sentence) and join it to the next state, which is either a word state or the initial states
        #of the phrases associated to the variable
        
        if("?" not in currentWord):  #if the current word is not a variable
            #print currentWord
            #print State
            #print ("#######make CA#####")
            #print (FSASentencesPop)
            #print (State)
            fsa.makeCA(FSASentencesPop,State)    ##make a CA for the current state (word) in the FSA Sentence population
            ##turn on word neuron
            #print wordsInSen
            wordNeuron=findWordNeuron(Words,currentWord)  # find the neuron corresponding to the currentword
                
            fsa.stateTurnsOnOneNeuron(FSASentencesPop,State,WordPop,wordNeuron) #turn on the word

                #turn on next state 
            if((eachWord+1)<NumWords):
                nextword=wordsInSen[eachWord+1] #next word in sentence
                if("?" not in nextword): #if next word is not a variable
                    NextState=findStateCA(PhraseSenIndex,StartStateCA,nextword) ##find the state CA associated to the nextword in a given sentence
                                                                                ##StartStateCA is the index of the first state of a given sentence 
                    #turn on next state (word)
                    fsa.stateTurnsOnState(FSASentencesPop,State,FSASentencesPop,NextState)
                    prevState=State
                    State=NextState

                    #if(eachWord>0):
                    fsa.stateTurnsOffState(FSASentencesPop,NextState,FSASentencesPop,prevState)
                else:
                    ##turn on next state, i.e. connect/turn on next phrase(s)
                    [prevStates,wordNeuron]=turnOnStatesAndWordsInPhrases(NumPhraseInfo,StoredSenAndPhrases,nextword,StartStateCA,PhraseSenIndex,State,WordPopInfo,PhraseCAs,currentWord,FSASentencesPop)
                    State=prevStates

        else:
            if(eachWord==-1):  #if the first word in the sentence is a variable
                #print State
                #print State
                fsa.makeCA(FSASentencesPop,int(State))  #create start state for a sentence that starts with a variable.  This state will trigger the sentence
                #print currentWord
                #Variable=currentWord
                nextword=currentWord
                currentWord=[]
                
            else:
                if((eachWord+1)<NumWords):
                    nextword=wordsInSen[eachWord+1] #next word in sentence
                #Variable=nextword

            if("?" not in nextword): ##if the nextword is not a variable
                                     ##turn on next state and word
                #print State
                #print nextword
                NextState=findStateCA(PhraseSenIndex,StartStateCA,nextword) #find the state CA associated to the next word in a given sentence
                for i in range(0,len(State)):
                    ##use the next word state to turn off the final states of the current phrase
                    fsa.stateTurnsOffState(FSASentencesPop,NextState,FSASentencesPop,State[i])
                    ##use the final states of the current phrase to turn on the next state, i.e. word
                    fsa.stateTurnsOnState(FSASentencesPop,State[i],FSASentencesPop,NextState)
                State=NextState

                
                
            else:
                #if the next word is a variable, ##turn on/connect next phrase(s) associated to the nextword variable
                [prevStates,wordNeuron]=turnOnStatesAndWordsInPhrases(NumPhraseInfo,StoredSenAndPhrases,nextword,StartStateCA,PhraseSenIndex,State,WordPopInfo,PhraseCAs,currentWord,FSASentencesPop)
                State=prevStates
            #print prevword
            #print currentWord

            #else:
                
        prevword=currentWord
        currentWord=nextword      
                       

    ##last word turns off last state (s)
    #print "currentWord"
    #print currentWord
    #print "prevword"
    #print prevword
    if("?" not in currentWord): #if current word in the sentence is not a variable, use the last word neuron to turn off the current sentence state
        fsa.oneNeuronTurnsOffState(WordPop, wordNeuron, FSASentencesPop, State)
    else:
        #if current word is a variable, use the last word neuron to turn off all the final states of the previous phrases
        for i in range(0,len(State)):
            fsa.oneNeuronTurnsOffState(WordPop, wordNeuron, FSASentencesPop, State[i])

def CalculateNumStates(StoredSenAndPhrasesInfo):
    ##calculate the number of states for each sentence template
    print ("##########CalculateNumStates#################")
    StoredSenAndPhrases=StoredSenAndPhrasesInfo[0] ##this contains the sentence templates and the different phrases
    NumPhrases=StoredSenAndPhrasesInfo[1] #this contains the number of phrases (including the variable heading) and sentence templates (including sentence heading)
    NumSentences=NumPhrases[0]-1 # number of sentence templates excluding the sentence heading
    NumPhrases=NumPhrases[1:]  #extract out the number of phrases for each variable
    NumPhraseVars=len(NumPhrases) # number of variables (phrase categories)
    Phrases=StoredSenAndPhrases[1:] ##extract out the phrases.  Each element (list) contains a phrase list associated to a different variable
    TotalNumStates=0

    sentencePr=StoredSenAndPhrases[0][1:] #extract out the sentence templates but take out sentence heading
    #print sentencePr
    #print NumSentences

    PhraseSenIndex=[[]]
    NumStatesEachSen=np.zeros(NumSentences) ##records the number of states for each sentence template
    #print NumStatesEachSen
    for eachsentence in range(0,NumSentences):  #each sentence template
        
        words=sentencePr[eachsentence].replace('.','') ##remove . from sentence
        words=words.split() #split the sentence into words
        NumWords=len(words)      ##the number of words in the sentence (including variables)
        for eachWord in range(0,NumWords):
            if("?" not in words[eachWord]):  #if the word is not a variable
                #if not a variable
                #print ("not a variable")
                TotalNumStates+=1           #add one to the total number of states
                PhraseSenIndex[eachsentence]+=[words[eachWord]]  ## add the word to the list
                NumStatesEachSen[eachsentence]+=1   #record the number states for each sentence
            else:
                #if word is a variable
                for eachPhraseSet in range(0,NumPhraseVars):  #for each phrase set (i.e. for each variable)
                    
                    PhraseSet=Phrases[eachPhraseSet] #extract phrase set for a particular variable but exclude the variable heading
                    #print PhraseSet
                    if(words[eachWord]==PhraseSet[0]):##find the phrase set associated to the variable name
                        if(eachWord==0):  #if the first word is a variable, we create a start state to trigger the sentence
                            TotalNumStates+=1
                            NumStatesEachSen[eachsentence]+=1
                            PhraseSenIndex[eachsentence]+=[" "] #this empty phrase corresponds to the start state of a sentence that begins with a variable
                        #print "PhraseSet"
                        #print PhraseSet
                        #print "words[eachWord]"
                        #print words[eachWord]                        
                        for eachPhrase in range(1,NumPhrases[eachPhraseSet]):  ##for each phrase
                            WordsInPhrase=PhraseSet[eachPhrase].split()
                            NumWordsInPhrase=len(WordsInPhrase)
                            TotalNumStates+=NumWordsInPhrase                      ##number of states for a phrase equals the number of words
                            NumStatesEachSen[eachsentence]+=NumWordsInPhrase
                            #print PhraseSet[eachPhrase]
                            #print NumPhrases[eachPhraseSet]
                            #print NumPhrases
                            #print "variable"
                            PhraseSenIndex[eachsentence]+=[PhraseSet[eachPhrase]+PhraseSet[0]]  ##record each phrase and tag it with the associated variable,
                                                                                                ##we tag it as we could have the same phrase associated to
                                                                                                ##different variables

            #print "total"
            #print TotalNumStates                
                            
        

        TotalNumStates+=1 #for a full stop
        NumStatesEachSen[eachsentence]+=1
        PhraseSenIndex[eachsentence]+=["."]
        PhraseSenIndex+=[[]]
    

    PhraseSenIndex.remove([])
    #print PhraseSenIndex
    #print NumStatesEachSen
    #print TotalNumStates

    return [TotalNumStates,PhraseSenIndex,NumStatesEachSen]

def TurnOnPhrase (phrase, fsa,PhraseCAsInfo,StoredSenAndPhrasesInfo,Variable):
    ##turn on a phrase in a specific group (Variable)

    phrase=phrase+Variable ##add the variable name to the phrase

    
    PhraseCAs=PhraseCAsInfo[0]
    PhraseCAIndex=PhraseCAsInfo[1]

    #find the phrase (CA index) in the phrase CA population
    PhraseIndex=findPhraseCA(PhraseCAIndex,phrase)

    spikeGenerators = createOneInput(fsa)
    firstSpikeGenerator =  spikeGenerators[0]


    fsa.turnOnStateFromSpikeSource(firstSpikeGenerator,PhraseCAs,PhraseIndex) #turn on Sentence Number

def TurnOnSentence(Sentence,SentenceFSAs,NumStatesEachSen,fsa,StoredSenAndPhrasesInfo):
    #turn on the initial state of a particular sentence
    SentenceOn=Sentence.replace('.',' .')
    sentenceIndex=[]
    StoredSentences=StoredSenAndPhrasesInfo[0][0][1:]
    NumSentences=StoredSenAndPhrasesInfo[1][0]-1
    #print "bob", StoredSentences
    #print NumSentences
    #print SentenceOn
    foundsentence=0

    for eachSentence in range(0,NumSentences):
        if(StoredSentences[eachSentence]==SentenceOn):
            sentenceIndex=eachSentence
            foundsentence=1
            break

    if(foundsentence==0):
        print ("ERROR: could not find sentence")
    
    #print StoredSentences
    #print NumSentences
    #print sentenceIndex
    ##turn on a sentence
    spikeGenerators = createOneInput(fsa)
    firstSpikeGenerator =  spikeGenerators[0]

    SentenceCA=sum(NumStatesEachSen[:sentenceIndex])
    #print SentenceCA

    fsa.turnOnStateFromSpikeSource(firstSpikeGenerator,SentenceFSAs,SentenceCA) #turn on Sentence Number

def stateStartsSentence(stateCells,stateNumber,Sentence,SentenceFSAs,NumStatesEachSen,fsa,StoredSenAndPhrasesInfo):
    #turn on the initial state of a particular sentence
    SentenceOn=Sentence.replace('.',' .')
    sentenceIndex=[]
    StoredSentences=StoredSenAndPhrasesInfo[0][0][1:]
    NumSentences=StoredSenAndPhrasesInfo[1][0]-1
    #print "bob", StoredSentences
    #print NumSentences
    #print SentenceOn
    foundsentence=0

    for eachSentence in range(0,NumSentences):
        if(StoredSentences[eachSentence]==SentenceOn):
            sentenceIndex=eachSentence
            foundsentence=1
            break

    if(foundsentence==0):
        print ("ERROR: could not find sentence")
    
    #print StoredSentences
    #print NumSentences
    #print sentenceIndex
    ##turn on a sentence
    SentenceCA=sum(NumStatesEachSen[:sentenceIndex])
    #print SentenceCA

    fsa.stateTurnsOnState(stateCells,stateNumber,SentenceFSAs,SentenceCA) #turn on Sentence Number
    fsa.stateTurnsOffState(SentenceFSAs,SentenceCA,stateCells,stateNumber)

def createPhraseCAs(StoredSenAndPhrasesInfo,fsa):
    #create a CA for each phrase

    StoredSenAndPhrases=StoredSenAndPhrasesInfo[0][1:] ##this contains lists of phrases, where each list contains a list of phrases associated
                                                       ##to a different variable
                                                       ##[0][0] contains the sentence templates
    NumPhrases=StoredSenAndPhrasesInfo[1][1:] #this contains the number of phrases associated to each variable.  The number also includes the variable headings
    
    PhraseCAIndex=[] #records which CA corresponds to which phrase
    NumPhraseVars=len(NumPhrases)
    #print StoredSenAndPhrases  
    #print NumPhrases

    AllPhraseVars=len(NumPhrases)  #number of variables
    NumberPhrases=sum(NumPhrases)-AllPhraseVars  #obtain the total number of phrases.  We take 1 off as the initial number includes the variable headings

    #print AllPhraseVars
    #print NumberPhrases

    PhraseCAs = createNeurons(NumberPhrases,fsa)

    for i in range(0,NumberPhrases):
        fsa.makeCA(PhraseCAs,i)     #make a CA for the initial state of each phrase.

    for eachPhraseSet in range(0,NumPhraseVars):
        PhraseSet=StoredSenAndPhrases[eachPhraseSet][1:] ##extract the phrases minus the variable heading
        for eachPhrase in range(0,NumPhrases[eachPhraseSet]-1):  ##-1 off as we have taken out the variable heading
            PhraseCAIndex+=[PhraseSet[eachPhrase]+StoredSenAndPhrases[eachPhraseSet][0]] ##attach the variable name to each phrase

    #the first list in StoredSenAndPhrases are the prototype sentences, the rest are phrases

    #print PhraseCAIndex

    return [PhraseCAs,PhraseCAIndex]    

def createFSASentences(WordCAs,fsa,PhraseCAs,StoredSenAndPhrasesInfo):
    ##create an FSA for each sentence

    [TotalNumStates,PhraseSenIndex,NumStatesEachSen]=CalculateNumStates(StoredSenAndPhrasesInfo) #calculate the number of states for each sentence template

    #print PhraseSenIndex

    FSASentencesPop = createNeurons(TotalNumStates,fsa) ##contains all of the sentence states

    NumSentences=StoredSenAndPhrasesInfo[1][0]-1  ##number of sentences in file

    Sentences=StoredSenAndPhrasesInfo[0][0][1:] ##extract the sentences excluding the sentence header



    StartStateCA=0 #initial state of the first sentence

    for eachSentence in range(0,NumSentences):

        Words=Sentences[eachSentence].split() ##

        ##create an FSA for the sentence
        CreateSentenceFSA(fsa,StartStateCA,FSASentencesPop,WordCAs,NumStatesEachSen[eachSentence],Words,PhraseCAs,StoredSenAndPhrasesInfo,PhraseSenIndex[eachSentence])
        #initial state of the next sentence
        StartStateCA+=NumStatesEachSen[eachSentence]
    

    #print "PhraseSenIndex"
    #print PhraseSenIndex

    return  [FSASentencesPop,NumStatesEachSen]
        

