"""
The environment connects with nest via files (words.txt and actions.txt).
It connects with spinnaker via eieio.

Expect a picture every 10 ticks.
Keep the most recent sentence, and add it in a word at a time every
50 ticks.
"""

import sys

#import for spike injection and reception
import spynnaker8.external_devices as externaldevices

from pyNN.spiNNaker import *

import time as t
popsLoadedOnBoard = False

#slow environment down by factor of 10
environmentSpeed = 10
FIRST_WORD_COMMAND = 15


def canSend(label, sender):
        print_condition.acquire()
        print ("can Send crh",sender, spikeQueue.popsLoadedOnBoard)
        print_condition.release()
        spikeQueue.popsLoadedOnBoard = spikeQueue.popsLoadedOnBoard -1
        

def gotWordInput(label, time, sender):
        print ("word Input",label,time,sender)
        #if (spikeQueue.fHaveCommandFromBoard):  #The queue actually works.
        #        print("lost word")
        neuron_id = sender[0]
        neuron_id = neuron_id + FIRST_WORD_COMMAND
        spikeQueue.setCommandFromBoard(neuron_id)

def gotInput(label, time, sender):
        print ("got Input",label,time,sender)
        neuron_id = sender[0]
        spikeQueue.setCommandFromBoard(neuron_id)

def gotClockInput(label, time, sender):
        #print "got Clock Input",label,time,sender
        neuron_id = sender[0]
        spikeQueue.spinnSetTicks(time)

def spinnBoardStarted(label,sender):
    global popsLoadedOnBoard
    print ("Environment gets signal board is loaded and running", sender)
    popsLoadedOnBoard = True

class spikeQueue:
    fHaveCommandFromBoard = False
    commandFromBoard = []
    spinnTime = 0

    @staticmethod
    def setCommandFromBoard(commandNum):
            spikeQueue.commandFromBoard = spikeQueue.commandFromBoard + [commandNum]
            spikeQueue.fHaveCommandFromBoard = True
            
    @staticmethod
    #Take the command off the queue.  If it's the last set fHaveCommandFromBoard
    def getCommandFromBoard():
            #print 'gcfb' , spikeQueue.commandFromBoard
            if (len (spikeQueue.commandFromBoard) == 0):
                    return -1
            else:
                    firstCommand = spikeQueue.commandFromBoard[0]
                    numberOfCommands = len (spikeQueue.commandFromBoard)
                    spikeQueue.commandFromBoard = spikeQueue.commandFromBoard[1:numberOfCommands]
                    if (len (spikeQueue.commandFromBoard) == 0):
                        spikeQueue.fHaveCommandFromBoard = False
                    return firstCommand

    #for spinnaker only
    @staticmethod
    def spinnSetTicks(newTicks):
            spikeQueue.spinnTime = newTicks

    def __init__(self,NumberWords):
        self.timesToSend = 2 #use this to get a finite number of input cycles.
                             #Need to comment back in in writeBWPicture

        self.timeTicks = 0
        self.numberWords = NumberWords

        
        self.wordKey = 0x70800
        print ("spinn 4 spike serve init")
        self.spikesToSpinnConn = externaldevices.SpynnakerLiveSpikesConnection(
                                local_port=12345,
                                send_labels=["wordSpikeToBoard"],
                                receive_labels=["actionFromBoard","wordFromBoard"])

        self.spikesToSpinnConn.add_start_callback(
                                "wordSpikeToBoard", spinnBoardStarted)
        self.spikesToSpinnConn.add_receive_callback("actionFromBoard",gotInput)
        self.spikesToSpinnConn.add_receive_callback("wordFromBoard",gotWordInput)
        #self.connection=externaldevices.SpynnakerLiveSpikesConnection(
                                #receive_labels=["actionFromBoard",
                                #"clockFromBoard"],local_port=12346,  
                                #send_labels=None)
        #self.connection.add_receive_callback("clockFromBoard",gotClockInput)


        #init sentence variables
        self.currentSentenceLeft = 0
        self.currentPosInSent = 0

    #for nest only
    def setTicks(self,newTicks):
            self.timeTicks = newTicks

    #for nest only
    def getTicks(self):
        return self.timeTicks

    def callbackFromBoard(self,packet):
        #print "fromBoardA ", packet._eieio_header._payload_base
        #print "fromBoardA ", packet._eieio_header._is_time
        #print "fromBoardA ", packet._data_reader
        reader = packet._data_reader
        action = reader.read_byte()
        #print "fromBoardB ",  action
        
        spikeQueue.setCommandFromBoard(action)
    
    def sendWord(self):
        global popsLoadedOnBoard
        if not popsLoadedOnBoard:
                return
        if self.currentSentenceLeft == 0:
            return

        print ("sw ", self.currentSentence[self.currentPosInSent])
        neuronToSpike = self.currentSentence[self.currentPosInSent]
        if (neuronToSpike == -1):
                neuronToSpike = self.numberWords+1 #This is last word + 1 to
                #start the parse state.

        if (neuronToSpike < -1):
                sleepTime = neuronToSpike*-1
                #It looks like 1.1 sleep is about 1.7 spinnaker seconds
                t.sleep(sleepTime)
        else:
                self.spikesToSpinnConn.send_spike("wordSpikeToBoard",
                        neuronToSpike,send_full_keys=True)

        #t.sleep(0.1)
        t.sleep(0.01)

        self.currentPosInSent += 1
        self.currentSentenceLeft -= 1
        return

    def increaseTimeTicks(self):
        self.timeTicks+=environmentSpeed
        print (self.timeTicks,"time")
        return

    def writeBWPicture(self,bitArray,length):
        global popsLoadedOnBoard

        if simulator == "nest":
                print (self.timeTicks,"time")
                self.timeTicks+=environmentSpeed
                return
        elif simulator != "spinnaker":
                print ("bad simulator writeBWPicture")
                return

        if not popsLoadedOnBoard:
                return

        if (length == 0):
            return
        
        for i in range (0,length):
            mess = bitArray[i]
            self.bwSpikesToSpinnConn.send_spike("bwVisionSpikeToBoard",
                        mess,send_full_keys=True)

        #self.timesToSend -= 1
        #t.sleep(0.01)

    def writeColourPicture(self,bitArray,length):
        global popsLoadedOnBoard
        if simulator == "nest":
                #print self.timeTicks,"time"
                #self.timeTicks+=10
                self.timeTicks+=environmentSpeed
                return
        elif simulator != "spinnaker":
                print ("bad simulator writeColourPicture")
                return

        if not popsLoadedOnBoard:
                return

        if (length == 0):
            return
        
        for i in range (0,length):
            mess = bitArray[i]
            self.colourSpikesToSpinnConn.send_spike("colourVisionSpikeToBoard",
                        mess,send_full_keys=True)
        
    def writeSentence(self,sentence,length):
        print (sentence)
        if self.currentSentenceLeft == 0:
            self.currentPosInSent = 0
            self.currentSentenceLeft = length
            self.currentSentence = sentence
            print ("new sentence sent")
        else:
            print ("trying to parse new sentence while parsing", self.currentSentenceLeft,self.currentSentence)

    def sendNestBump(self):
        fileHandle = open("bumps.txt",'a')
        sendTime = self.timeTicks/10
        #sendTime = (sendTime*10)+40
        sendTime = (sendTime*10)+30
        outString = str(sendTime)
        outString += " 0\n"
        fileHandle.write(outString)

    def sendBump(self):
        #print "toss bump"
        if (nealParameters.simulator == "spinnaker"):
                mess = 0
                self.bumpSpikesToSpinnConn.send_spike("bumpSpikeToBoard",
                        mess,send_full_keys=True)
        elif (nealParameters.simulator == "nest"):
                self.sendNestBump()
